/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <gdt.h>
#include <idt.h>
#include <task.h>

struct GDTgate_t *GDTgates=(struct GDTgate_t *)0x3200;/*(struct GDTgate_t *)(IDT_SIZE*sizeof(struct IDTgate_t));*/
struct GDTgate_t LDTgates[2];

byte GDTgatesMap[GDT_SIZE]={0,};

struct gdtr_t{
  word limit __attribute__ ((packed));
  dword base __attribute__ ((packed));
}gdtr; /* assure to be as little as possible */

static void setGDTR(dword base, word limit){
   CLI();
   gdtr.limit=limit;
   gdtr.base=(dword)base;

   __asm__ __volatile__ ("lgdt (%0)": :"r" (&gdtr));
   STI();
}

void initGDTgatesMap(void){
   int i;
   for(i=0;i<GDT_SIZE;i++)
      GDTgatesMap[i]=0;
}

void initLDTgates(void){
   LDTgates[0] = makeGDTgate(MAX_SEG_SIZE, 0x100000, GDT_G_4KB, GDT_CODE_ER, GDT_DPL3);
   LDTgates[1] = makeGDTgate(MAX_SEG_SIZE, 0x100000, GDT_G_4KB, GDT_DATA_RW, GDT_DPL3);
}

void initGDT(void){
   initGDTgatesMap();
   initLDTgates();

   /*GDT descriptors*/
   setGDTgate(0,makeNullGDTgate());
   setGDTgate(1,makeGDTgate(MAX_SEG_SIZE, 0x0, GDT_G_4KB, GDT_CODE_ER, GDT_DPL0));
   setGDTgate(2,makeGDTgate(MAX_SEG_SIZE, 0x0, GDT_G_4KB, GDT_DATA_RW, GDT_DPL0));
   setGDTgate(GLOBAL_LDT,makeGDTgate((sizeof(struct GDTgate_t)*2)-1, (dword)(&LDTgates), GDT_G_1B, GDT_LDT, GDT_DPL3));

   /*setting register*/
   setGDTR((dword)GDTgates,(GDT_SIZE*sizeof(struct GDTgate_t))-1);
}

struct GDTgate_t makeNullGDTgate(void){
   struct GDTgate_t entry;
   *((dword*)&entry) = *(((dword*)&entry)+1) = 0;
  return entry;
}

struct GDTgate_t makeGDTgate(dword limit, dword base, byte G, byte type, byte DPL){
   struct GDTgate_t entry;

   entry.LimitL = limit&0xFFFF;
   entry.BaseL = base&0xFFFF;
   entry.BaseHL = (base&0xFF0000)>>16;
   entry.Options.Type = type&0xF;
   entry.Options.S = (type&0x10)>>4;
   entry.Options.DPL = DPL;
   entry.Options.Present = 1;
   entry.Options.LimitH = (limit&0xF0000)>>16;
   entry.Options.AVL = 1;
   entry.Options.L = 0;
   entry.Options.DB = 1;
   entry.Options.Granularity = G;
   entry.BaseHH = (base&0xFF000000)>>24;

  return entry;
}

int getFreeGDTgate(void){
   int i;
   for(i=0;i<GDT_SIZE;i++){
      if(GDTgatesMap[i]!=GDT_USED){
         GDTgatesMap[i]=GDT_USED;
         return i;
      }
   }
   return -1;
}

void freeGDTgate(int i){
   if(i<GDT_SIZE)
      GDTgatesMap[i]=0;
}

int setGDTgate(int idx, struct GDTgate_t gate){
   if(idx<GDT_SIZE){
      GDTgatesMap[idx]=GDT_USED;
      GDTgates[idx] = gate;
      return idx;
   }
   return -1;
}
