/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <idt.h>
#include <syscall.h>
#include <task.h>
#include <scheduler.h>
#include <mem.h>

struct IDTgate_t *IDTgates = (struct IDTgate_t*)(0x3000);
void (*INTtable[IDT_SIZE])();


void nullIntHandler(void);
void exceptionHandler();
void debugIntHandler(void);

struct idtr_t{
  word limit __attribute__ ((packed));
  dword base __attribute__ ((packed));
}idtr; /* assure to be as little as possible */

void setIDTR(dword base, word limit){
   idtr.limit=limit;
   idtr.base=(dword)base;

   __asm__ __volatile__ ("lidt (%0)": :"r" (&idtr));
}

struct IDTgate_t makeIDTgate(void (*func)(), int DPL, word type){
   struct IDTgate_t tmp;
   word codesegment = 0x08;

   __asm__ volatile("movw %%cs,%0":"=g" (codesegment));

   tmp.OffsetL = (((dword)func) & 0x0000FFFF); /* 0-15bit of the address */
   tmp.OffsetH = (((dword)func) >> 16); /* 16-31 bit of the address */
   tmp.Selector = codesegment; /* use this selector */
   tmp.Options.Reserved0 = 0x00;
   tmp.Options.Reserved1 = type;
   tmp.Options.Reserved2 = 0x1;
   tmp.Options.Reserved3 = 0x0;
   tmp.Options.SizeD = 0x1;
   tmp.Options.DPL = DPL;
   tmp.Options.Present = 0x1;
    /*tmp.Options = type;*/

   return tmp;
}

extern void SYSCALL_ISR(void);

void initIDTgates(void){
   int i;

   IDTgates[0] = makeIDTgate(ISR_0, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[1] = makeIDTgate(ISR_1, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[2] = makeIDTgate(ISR_2, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[3] = makeIDTgate(ISR_3, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[4] = makeIDTgate(ISR_4, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[5] = makeIDTgate(ISR_5, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[6] = makeIDTgate(ISR_6, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[7] = makeIDTgate(ISR_7, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[8] = makeIDTgate(ISR_8, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[9] = makeIDTgate(ISR_9, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[10] = makeIDTgate(ISR_10, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[11] = makeIDTgate(ISR_11, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[12] = makeIDTgate(ISR_12, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[13] = makeIDTgate(ISR_13, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[14] = makeIDTgate(ISR_14, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[15] = makeIDTgate(ISR_15, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */
   IDTgates[16] = makeIDTgate(ISR_16, KERNEL_MODE,INTERR_GATE); /*EXCEPTION */

   /* UNUSED */
   for(i=17;i<=31;i++)
      IDTgates[i] = makeIDTgate(ISR_NULL, KERNEL_MODE,INTERR_GATE); /*RESERVED*/

   /* USER DEFINED */
   IDTgates[32] = makeIDTgate(ISR_32, KERNEL_MODE,INTERR_GATE); /*IRQ 0*/
   IDTgates[33] = makeIDTgate(ISR_33, KERNEL_MODE,INTERR_GATE); /*IRQ 1 (keyboard)*/
   IDTgates[34] = makeIDTgate(ISR_34, KERNEL_MODE,INTERR_GATE); /*IRQ 2*/
   IDTgates[35] = makeIDTgate(ISR_35, KERNEL_MODE,INTERR_GATE); /*IRQ 3*/
   IDTgates[36] = makeIDTgate(ISR_36, KERNEL_MODE,INTERR_GATE); /*IRQ 4*/
   IDTgates[37] = makeIDTgate(ISR_37, KERNEL_MODE,INTERR_GATE); /*IRQ 5*/
   IDTgates[38] = makeIDTgate(ISR_38, KERNEL_MODE,INTERR_GATE); /*IRQ 6*/
   IDTgates[39] = makeIDTgate(ISR_39, KERNEL_MODE,INTERR_GATE); /*IRQ 7*/
   IDTgates[40] = makeIDTgate(ISR_40, KERNEL_MODE,INTERR_GATE); /*IRQ 8*/
   IDTgates[41] = makeIDTgate(ISR_41, KERNEL_MODE,INTERR_GATE); /*IRQ 9*/
   IDTgates[42] = makeIDTgate(ISR_42, KERNEL_MODE,INTERR_GATE); /*IRQ 10*/
   IDTgates[43] = makeIDTgate(ISR_43, KERNEL_MODE,INTERR_GATE); /*IRQ 11*/
   IDTgates[44] = makeIDTgate(ISR_44, KERNEL_MODE,INTERR_GATE); /*IRQ 12*/
   IDTgates[45] = makeIDTgate(ISR_45, KERNEL_MODE,INTERR_GATE); /*IRQ 13*/
   IDTgates[46] = makeIDTgate(ISR_46, KERNEL_MODE,INTERR_GATE); /*IRQ 14*/
   IDTgates[47] = makeIDTgate(ISR_47, KERNEL_MODE,INTERR_GATE); /*IRQ 15*/
   IDTgates[48] = makeIDTgate(SYSCALL_ISR, USER_MODE, TRAP_GATE); /*SYSCALLs int 0x30*/
}

void initIntTable(void){
   int i;
   for(i=0;i<20;i++)
      INTtable[i]=(void(*)())exceptionHandler;
   for(i=20;i<IDT_SIZE;i++)
      INTtable[i]=nullIntHandler;
}

void initIDT(void){
   initIntTable();
   initIDTgates();
   setIDTR((dword)IDTgates,(IDT_SIZE*sizeof(struct IDTgate_t))-1);
   STI(); /* enable interrupts */
}


/* Interrupt Descriptor table.
   Interupt Service Routine.

   These assembly routines simply
   move to kernel mode and wraps
   to INTtable[i] vector table 
*/
#define MOVE_KERNEL_MODE\
         "movl $0x10,%edx;\
          movl %edx,%ds;\
          movl %edx,%es;"

#define STARTUP\
         "pusha;\
          pushw %ds;\
          pushw %es;\
          "\
          MOVE_KERNEL_MODE

#define FINALIZE\
         "popw %es;\
          popw %ds;\
          popa;"

#define ISR(n)\
   __asm__("ISR_"#n":\
             "\
             STARTUP\
             "\
             call *(INTtable+(4*"#n"));\
             "\
             FINALIZE\
             "\
             iret;\n\
          ")

#define EXC(n)\
   __asm__("ISR_"#n":\
             "\
             STARTUP\
             "\
             pushl $"#n";\
             call *(INTtable+(4*"#n"));\
             addl $4, %esp;\
             "\
             FINALIZE\
             "\
             iret;\n\
          ")

/*
   STACK IMAGE:

      from kmode
         +EFLAGS
         +CS
         +EIP
         +Err Code

      from umode
         +SS
         +ESP
         +EFLAGS
         +CS
         +EIP
         +Err Code
*/
#define EXC_ECODE(n)\
   __asm__("ISR_"#n":\
             \
             pushl %eax;\
             movl 4(%esp), %eax;\
             pushl %ebx;\
             pushl %ecx;\
             pushl %edx;\
             pushl %ebp;\
             pushl %esi;\
             pushl %edi;\
             pushw %ds;\
             pushw %es;\
             "\
             MOVE_KERNEL_MODE\
             "\
             pushl %eax;\
             pushl $"#n";\
             call *(INTtable+(4*"#n"));\
             addl $8, %esp;\
             \
             popw %es;\
             popw %ds;\
             popl %edi;\
             popl %esi;\
             popl %ebp;\
             popl %edx;\
             popl %ecx;\
             popl %ebx;\
             popl %eax;\
             addl $4, %esp;\
             iret;\n\
          ")

/*
   EXCEPTIONS ISR
*/
EXC(0);  EXC(1);  EXC(2);
EXC(3);  EXC(4);  EXC(5);
EXC(6);  EXC(7);  EXC(8);
EXC(9);  EXC(10); EXC(11);
EXC(12); EXC(13); EXC_ECODE(14);
EXC(15); EXC(16); EXC(17);
EXC(18); EXC(19);

/*
  20-31 are Intel Reserved
*/
__asm__("ISR_NULL:\
           jmp ISR_NULL;\
           iret\n\
        "); 

/* TIMER INTERRUPT
   SERVICE ROUTINE
*/
__asm__("ISR_32:\
           "
           STARTUP
           "\
           pushl $ret_from_int;\
           call *(INTtable+(4*32));\
           \
           movb $0x20,%al;\
           outb %al,$0x20;\
           \
           ret\n\
        ");

/*
   IRQs ISR
*/
ISR(33); ISR(34); ISR(35);
ISR(36); ISR(37); ISR(38);
ISR(39); ISR(40); ISR(41);
ISR(42); ISR(43); ISR(44);
ISR(45); ISR(46); ISR(47);

/* RETURN FROM INTERRUPT
   SERVICE ROUTINE
   It allows process switch
*/
__asm__("ret_from_int:\
           cli;\
           call do_schedule;\
           "
           FINALIZE
           "\
           iret\n\
       ");

/*
   EXCEPTION C HANDLER
*/
extern
void do_exit(int);

extern
void on_page_fault(dword address, dword code);

void
exceptionHandler(dword num, dword code)
{
   char *desc;
   switch(num)
   {
      case 0: {desc = "Divide Error Exception (#DE)"; break;}
      case 1: {desc = "Debug Exception (#DB)"; break;}
      case 2: {desc = "NMI interrupt"; break;}
      case 3: {desc = "Breakpoint Exception (#BP)"; break;}
      case 4: {desc = "Overflow Exception (#OF)"; break;}
      case 5: {desc = "Range Exceeded Exception (#BR)"; break;}
      case 6: {desc = "Invalid Opcode Exception (#UD)"; break;}
      case 7: {desc = "Device Not Avalilable Exception (#NM)"; break;}
      case 8: {desc = "Double Fault Exception (#DF)"; break;}
      case 9: {desc = "Coprocessor Segment Overrun"; break;}
      case 10: {desc = "Invalid TSS Exception (#TS)"; break;}
      case 11: {desc = "Segment Not Present (#NP)"; break;}
      case 12: {desc = "Stack Fault Exception (#SS)"; break;}
      case 13: {desc = "General Protection Exception (#GP)"; break;}
      case 14: {
         dword request;
         desc = "Page Fault Exception (#PF)";
         /* the page fault source address*/
         __asm__("mov %%cr2,%%eax":"=a"(request));
         on_page_fault(request, code);

         return;
      }
      case 15: {desc = "RESERVED"; break;}
      case 16: {desc = "FPU Floating-Point Error (#MF)"; break;}
      case 17: {desc = "Alignment Check Exception (#AC)"; break;}
      case 18: {desc = "Machine Check Exception (#AC)"; break;}
      case 19: {desc = "SIMD Floating Exception (#AC)"; break;}
      default: {desc = "Unknown Exception"; break;}
   }
   if(!ON_STARTUP()){
      kprintf("Process pid=%d: %s!\n",GET_CURRENT_TASK()->Info.pid,desc);
      do_exit(-1);
   }
   else{
      kprintf("Kernel Panic! Exception #%u caught: <%s>\n",num,desc);
      for(;;);
  }
  return;
}

void
nullIntHandler(void)
{
   kprintf("\nKernel panic: the system has been halted!\n");
   for(;;);
  return;
}

/*
   NOTE:
   If you're asking:
   "WHY DIDN'T YOU MAKE A ASM FILE INSTEAD OF THIS?"

   I can't answer that question...
   ...maybe I like ansi C preprocessor too much...:D
*/
