/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <fs.h>
#include <std/string.h>


#define strTolower(str) ({\
   while(*str){\
      *str = tolower(*str);\
       str++;}\
   str;\
   })

/* binary image file 
(refer f2c & mkfs utility) */

struct bin_table_t{
   char name[64];
   unsigned char* data;
   unsigned int size;
};
extern struct bin_table_t bin_table[];


static byte *path2bin(char *path, int *size){
   int i;

   if(!(*path))
      return NULL;

   for(i=0;bin_table[i].name[0];i++){
      if(strcmp(bin_table[i].name,path)==0){
         if(size) *size = bin_table[i].size;
         return (byte*)bin_table[i].data;
      }
   }

   return NULL;
}

file_t fs_open(char *path){
   file_t tmp;

   tmp.size = 0;
   tmp.base = tmp.offset = NULL;

   tmp.base = tmp.offset = (char *)path2bin(path, &tmp.size);

/*
   if(strcmp("shell.bin",path)==0){
      tmp.base = tmp.offset = SHELL_BIN;
      tmp.size = sizeof(SHELL_BIN);
   }
*/
   return tmp;
}

int fs_read(file_t *file, char *buffer, int size){
   if(file && buffer && size>0 && file->offset < file->base+file->size){

      if((file->offset + size) >= file->base + file->size)
         size = file->base + file->size - file->offset - 1;

      memcpy(buffer, file->offset, size);
      file->offset += size;
      return size;
   }
   return 0;
}

int fs_close(file_t *file){
   file->base = NULL;
   return 0;
}

int fs_seek(file_t *file, int offset){
   return (int)(file->offset+=offset);
}

int fs_exists(char *path, int *size){
   return (int)path2bin(path, size);
}
