/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef _MEM_H_
#define _MEM_H_

#include<types.h>
#include<std/asm.h> /*PAGE_SIZE*/

/*MB*/
/*#define VIRT_MEM_SIZE 4096
#define PAGE_DIR_ADDR 0x100000
*/
#define getPage()\
   getPages(1)

#define freePage(p)\
   freePages(p,1)

#define IS_USER_FAULT(code)\
   ((code|0x4)==code)

#define loadPageDirRegister(pdbr)\
   __asm__ volatile("\
                     mov %0, %%eax;\
                     mov %%eax, %%cr3;\
                    "::"r"(pdbr));

/* Warning: ISO C forbids
   braced-groups within
   expressions
*/
#define storePageDirRegister()\
   ({\
     unsigned int pdbr;\
     __asm__ volatile("mov %%cr3,%0":"=r"(pdbr));\
     pdbr;\
    })

/*
  make cr3 register point to
  PDBP input value
 */
#define enablePaging()\
   __asm__ volatile("\
                 pushl %eax;\
                 mov %cr0,%eax;\
                 or $0x80000000,%eax;\
                 mov %eax, %cr0;\
                 popl %eax;\
               ")

/*
  disable paging and get old
  PDBP value
 */
#define disablePaging()\
       __asm__ volatile("\
                 pushl %eax;\
                 mov %cr0,%eax;\
                 and $0x7FFFFFFF,%eax;\
                 mov %eax, %cr0;\
                 popl %eax;\
               ")

#define isEnabledPaging()\
   ({unsigned int cr0; __asm__ volatile ("mov %%cr0,%0":"=r"(cr0));cr0>>31;})


#define PAGE_DIR_ENTRY(P, US, PageTableNo)\
   ({\
     PageDirEntry dir;\
     dir.Present = P;\
     dir.ReadWrite=1;\
     dir.UserSupervisor=US;\
     dir.WriteThrough=1;\
     dir.ChacheDisabled=0;\
     dir.Accessed=0;\
     dir.Reserved=0;\
     dir.PageSize=0;\
     dir.GlobalPage=0;\
     dir.Available=0;\
     dir.PageTableBaseAddress=PageTableNo;\
     dir;\
   })

#define PAGE_TABLE_ENTRY(P, US, PageNo)\
   ({\
      PageTableEntry table;\
      table.Present = P;\
      table.ReadWrite=1;\
      table.UserSupervisor=US;\
      table.WriteThrough=1;\
      table.ChacheDisabled=1;\
      table.Accessed=0;\
      table.Dirty=0;\
      table.PageTableAttributeIndex=0;\
      table.GlobalPage=0;\
      table.Available=0;\
      table.PageBaseAddress=PageNo;\
      table;\
   })

/*PAGE_SIZE is defined in std/asm.h*/
/*
#define PAGE_SIZE 4096
*/

struct pageDirEntry_t{
   dword Present:1;
   dword ReadWrite:1;
   dword UserSupervisor:1;
   dword WriteThrough:1;
   dword ChacheDisabled:1;
   dword Accessed:1;
   dword Reserved:1;
   dword PageSize:1;
   dword GlobalPage:1;
   dword Available:3;
   dword PageTableBaseAddress:20;   
}__attribute__((packed));

typedef struct pageDirEntry_t PageDirEntry;

struct pageTableEntry_t{
   dword Present:1;
   dword ReadWrite:1;
   dword UserSupervisor:1;
   dword WriteThrough:1;
   dword ChacheDisabled:1;
   dword Accessed:1;
   dword Dirty:1;
   dword PageTableAttributeIndex:1;
   dword GlobalPage:1;
   dword Available:3;
   dword PageBaseAddress:20;
}__attribute__((packed));

typedef struct pageTableEntry_t PageTableEntry;

/* memory numbers */
#define KERNEL_MEM_LIMIT 0x100000
#define KERNEL_PAGE_SIZE (KERNEL_MEM_LIMIT >> 12)
/* 8 MB == 800000h B */
#define PHYS_MEM_LIMIT 0x800000
#define PHYS_PAGE_SIZE (PHYS_MEM_LIMIT >> 12)
#define FREE_PAGES_COUNT (PHYS_PAGE_SIZE - KERNEL_PAGE_SIZE)
#define PAGES_BITMAP_SIZE (FREE_PAGES_COUNT >> 3)

#define KERNEL_CACHE 0x0
#define KERNEL_CACHE_SIZE (PAGE_SIZE*2)

#define DIR(address) (address>>22)
#define TABLE(address) ((address & 0x003FF000)>>12)
#define OFFSET(add) (address & 0x00000FFF)

/*
 * Move to physical addressing
 */
dword BEGIN_PHYS_ADDR(int dis_int);
/*
 * Move to paged addressing
 */
void END_PHYS_ADDR(dword iesp, int en_int);

void initMem(void);
void *getPages(int n);
void freePages(void *page, int n);

#define PAGE_TYPE_KERNEL 0x0
#define PAGE_TYPE_EMPTY  0x1
#define PAGE_MODE_SYSTEM 0x0
#define PAGE_MODE_USER   0x1
#define PAGE_ENTRY_NOT_PRESENT 0x0
#define PAGE_ENTRY_PRESENT 0x1

PageTableEntry *create_page_table(PageTableEntry*, int type, int mode);
PageDirEntry *create_page_dir(PageDirEntry *, PageTableEntry *);
void share_task_struct(int pid, void *address);
void demandPage(dword address, dword code);
void callback_pages_for(int pid);

#endif
