/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef _TASK_H_
#define _TASK_H_

#include <types.h>
#include <kernel.h>
#include <mem.h>
#include <gdt.h>
#include <task.h>
#include <scheduler.h>
#include <lock.h>

#define TASKS_MAX_NUM 1024

#define GET_CURRENT_TASK()\
   ({\
      union task_t *task;\
      __asm__ volatile("movl %%esp,%0":"=r"(task));\
      task=(union task_t *)((dword)task&0xFFFFF000);\
    })

extern union task_t *tasksTable[];
extern int tasksCount;

#define task2pid(t)\
   (t->pid >=0 && t->pid <TASKS_MAX_NUM) ? t->pid : -1

#define pid2task(pid)\
   (union task_t *)((pid<TASKS_MAX_NUM) ? ((tasksTable[pid]) ? tasksTable[pid] : NULL ) : NULL)

#define TASK_STRUCT_ADDRESS(pid)\
   (0xFFFFF000-(pid<<12))

#define USER_MODE_STACK 0xFFAFFFFF

#define KERNEL_MODE_STACK(pid)\
   (TASK_STRUCT_ADDRESS(pid)+0xFFF)

struct TSS_t{
   word PreviousTask, _PreviousTask;
   dword ESP0;
   word SS0, _SS0;
   dword ESP1;
   word SS1, _SS1;
   dword ESP2;
   word SS2, _SS2;
   dword CR3;
   dword EIP;
   dword EFLAGS;
   dword EAX;
   dword ECX;
   dword EDX;
   dword EBX;
   dword ESP;
   dword EBP;
   dword ESI;
   dword EDI;
   word ES, _ES;
   word CS, _CS;
   word SS, _SS;
   word DS, _DS;
   word FS, _FS;
   word GS, _GS;
   word LDT, _LDT;
   word T, IOMAP;
}__attribute__((packed));


enum status_t{READY, RUNNING, STOPPED, EXITING, TERMINATED};

union task_t{
   struct{

      struct{
         struct TSS_t TSS;
         struct GDTgate_t LDT[2];
         int tssIdx;
         int ldtIdx;
         PageDirEntry *pageDir;
      }System;

      struct{
         int dynamic;
         int nice;
      }Priority;

      struct{
         union task_t *physical;
         union task_t *paged;
         unsigned char *base;
         int size;
         int pages;
      }Address;

      struct{
         union task_t *next;
         union task_t *prev;
      }Link;

      char name[64];
      enum status_t status;
      int pid;

   }Info;

   byte Stack[PAGE_SIZE];
};

void initTasks(void);

int tsk_create_process(char *path, int argc, char *argv[]);
void destroyTask(int pid);

#endif
