/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <timer.h>
#include <std/vsprintf.h>

#define PIT8253_HERTZ 1193182L
#define TIMER_COUNT (PIT8253_HERTZ / TIMER_HERTZ)

#define CMOS_GET_SEC   0x0 
#define CMOS_GET_MIN   0x2
#define CMOS_GET_HOUR  0x4
#define CMOS_GET_DAY   0x7
#define CMOS_GET_MONTH 0x8
#define CMOS_GET_YEAR  0x9

static unsigned char readCMOS(unsigned char addr){
   outport_wait(0x70,addr);
   return inport_wait(0x71);
}
static void writeCMOS(unsigned char addr, unsigned int value){
   outport_wait(0x70, addr);
   outport_wait(0x71, value);
}

void timerHandler(void){
}

void initTimer(void){
   outport_wait(0x43, 0x36);
   outport_wait(0x40, (byte)(TIMER_COUNT & 0xFF));
   outport_wait(0x40, TIMER_COUNT>>8);

   setHandlerIRQ(TIMER_IRQNO, timerHandler);
}

#define BCD2BIN(bcd) ((((bcd)&15) + ((bcd)>>4)*10))
datetime_t getDatetimeNow(void){
   datetime_t now;

   cli();
   now.sec = BCD2BIN(readCMOS(CMOS_GET_SEC));
   now.min = BCD2BIN(readCMOS(CMOS_GET_MIN));
   now.hour = BCD2BIN(readCMOS(CMOS_GET_HOUR));
   now.day = BCD2BIN(readCMOS(CMOS_GET_DAY));
   now.month = BCD2BIN(readCMOS(CMOS_GET_MONTH));
   now.year = BCD2BIN(readCMOS(CMOS_GET_YEAR));
   sti();

   return now;
}


char *datetimeToString(datetime_t now, char *str){
   sprintf(str,"%02d:%02d:%02d %02d/%02d/%02d",
              now.hour, now.min, now.sec, now.day, now.month, now.year);
   return str;
}

datetime_t stringToDatetime(char *str){
   datetime_t dt;
   /*TODO*/
   dt.hour = dt.min = dt.sec = dt.day = dt.month = dt.year = 0;
   return dt;
}
