/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <scheduler.h>
#include <mem.h>

volatile dword ready_to_schedule = 0;

/* segment selector */
volatile dword switch_addr[2]={0,0};

void contextSwitch(void){
   int tss, pid;
   /* get tss of the next task */
   if(!ON_STARTUP()){
      tss = GET_CURRENT_TASK()->Info.Link.next->Info.System.tssIdx;
      pid = GET_CURRENT_TASK()->Info.Link.next->Info.pid;
   }
   else{
      tss = 0x4;
      pid = 0;
   }

   callback_pages_for(pid);

   /* make the target segment selector RPL=3 */
   switch_addr[0] = 0;
   switch_addr[1] = SEG_SEL(tss,TI_GDT,GDT_DPL3);

   /* do jmp */
   __asm__ volatile("ljmp *%0"::
               "m"(switch_addr[0]),"m"(switch_addr[1]):
               "memory");
}

void do_schedule(void)
{
      union task_t *current;
      union task_t *next;

      if(ON_STARTUP())
         return;

      current = GET_CURRENT_TASK();
      next = current->Info.Link.next;

      if(next == current){
         return; /*no schedule needed, last process cannot exit */
      }
      else if(current->Info.status == EXITING){
         current->Info.status = TERMINATED;
         destroyTask(current->Info.pid);
         contextSwitch();
         return;
      }
      else if(--(current->Info.Priority.dynamic)<=0){
         current->Info.Priority.dynamic = QUANTUM;
         current->Info.status = STOPPED;
         next->Info.status = RUNNING;
         contextSwitch();
         return;
      }
      else{
         return;
      }
}
