/*

This file is part of aiorOS

aiorOS - 32bit Operating System.
Copyright (C) 2005  Salvatore Iengo <salvatoreiengo@hotmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <std/stddef.h>
#include <syscall.h>

/* system calls table */
void *syscall_table[]={
   &sys_write,
   &sys_read,
   &sys_create_process,
   &sys_create_thread,
   &sys_exit,
   &sys_wait
};

int SYSCALL_COUNT=sizeof(syscall_table)/sizeof(void*);

#define USER_SPACE(p)\
   (((byte*)(GET_CURRENT_TASK()->Info.Address.base))+(dword)p)

#define CHECK_BOUNDARIES(p)\
   ((((byte*)p>=(byte*)(GET_CURRENT_TASK()->Info.Address.base)) &&\
     ((byte*)p<(byte*)(((GET_CURRENT_TASK()->Info.Address.base)+(GET_CURRENT_TASK()->Info.Address.size))))) ?\
      (byte*)p : (byte*)-1)

#define USER_SPACE_FINE(p)\
   CHECK_BOUNDARIES(USER_SPACE_UNCHECK(p))

/* SYSTEM CALL - write -*/
int sys_write(void *arg, int count){
   int fd;
   char* buffer;
   int bufferLen;
   int i=-1;

   if(count !=3 || arg==0)
      return -1;

   if((arg = USER_SPACE(arg))<0)
      return -1;

   fd = (int)((void**)arg)[0];

   if((buffer = (char*)USER_SPACE(((void**)arg)[1]))<0)
      return -1;

   bufferLen = (int)(((void**)arg)[2]);

   if(fd==STDOUT){
      ttio_write(buffer, bufferLen);
   }

  return i;
}

int sys_read(void *arg, int count){
   int fd;
   char* buffer;
   int bufferLen;
   int i=-1;

   if(count !=3 || arg==0)
      return -1;

   if((arg = USER_SPACE(arg))<0)
      return -1;

   fd = (int)((void**)arg)[0];

   if((buffer = (char*)USER_SPACE(((void**)arg)[1]))<0)
      return -1;

   bufferLen = (int)(((void**)arg)[2]);

   if(fd==STDIN){
      i = ttio_read(buffer, bufferLen);
   }

  return i;
}

int sys_create_process(void *arg, int count){
   char *path;
   int i;
   void *argv[32];

   if(count < 1)
      return -1;

   if((arg = USER_SPACE(arg))<0)
      return -1;

   if((path = (char*) USER_SPACE(((void**)arg)[0]))<0)
      return -1;

   for(i=1;i<count;i++){
      if((argv[i-1] = USER_SPACE(((void**)arg)[i]))<0)
         return -1;
   }

   return tsk_create_process((char*)path, count-1, (char**)argv);
}

int sys_create_thread(void *arg, int count){return -1;}

/* TODO: handle return status */
int sys_exit(void *arg, int count)
{
   union task_t *current = GET_CURRENT_TASK();

   /*last process cannot exit*/
   if(current->Info.Link.next == current){
      return -1;
   }

   current->Info.status = EXITING;
   return 0;
}

/* TODO: make it unix similar */
int sys_wait(void *arg, int count){
   int pid;

   if((arg = USER_SPACE(arg))<0)
      return -1;

   if(count!=1)
      return -1;

   pid = (int)(((void**)arg)[0]);

   if(!pid2task(pid) || pid==(GET_CURRENT_TASK()->Info.pid))
      return -1;
   else
      while(pid2task(pid));

  return 0;
}

__asm__(".global SYSCALL_ISR;\
      .align 16,0x90;\
        SYSCALL_ISR:\
        \
        push %esp;\
        push %ebp;\
        pushw %ds;\
        pushw %es;\
        pushw %fs;\
        pushw %gs;\
        movl $0x10,%edx;\
        movl %edx,%ds;\
        movl %edx,%es;\
        \
        pushl $ret_from_sys;\
        \
        cmpl SYSCALL_COUNT,%eax;\
        jae _end;\
        \
        pushl %ecx;\
        pushl %ebx;\
        call *syscall_table(,%eax,4);\
        addl $4*2,%esp;\
        \
        _end:\
        ret;\
   ");

/* eax contains syscall
   return value...
*/
__asm__("ret_from_sys:\
        cli;\
        pushl %eax;\
        call do_schedule;\
        popl %eax;\
        popw %gs;\
        popw %fs;\
        popw %es;\
        popw %ds;\
        pop %ebp;\
        pop %esp;\
        iret;\
    ");
